# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information
import datetime
import os

on_rtd = os.environ.get("READTHEDOCS", None) == "True"

try:
    import sphinx_rtd_theme

    html_theme = "sphinx_rtd_theme"
    # html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
except ImportError:
    html_theme = "default"
    if not on_rtd:
        print("-" * 74)
        print("Warning: sphinx-rtd-theme not installed, building with default theme.")
        print("-" * 74)

source_suffix = ".rst"
master_doc = "index"

project = "icalendar_compatibility"
this_year = datetime.date.today().year
copyright = f"{this_year}, Nicco Kunzmann"
author = "Nicco Kunzmann"
release = "v0.0.1"

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.coverage",
    "sphinx.ext.viewcode",
    "sphinx_copybutton",
    "sphinx.ext.intersphinx",
    "sphinx.ext.autosectionlabel",
]

intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    "icalendar": ("https://icalendar.readthedocs.io/en/latest/", None),
}

templates_path = ["_templates"]

exclude_patterns = ["_build", "lib", "bin", "include", "local"]
pygments_style = "sphinx"


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

# html_theme = "alabaster"
# html_static_path = ["_static"]

add_module_names = False



# icalendar_compatibility
# Copyright (C) 2025  Nicco Kunzmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see
# <https://www.gnu.org/licenses/>.
"""Event description computation."""

import re
from urllib.parse import unquote

from icalendar import Event, vText

RE_HTML_TAGS_PROPER = re.compile(r"^([^<]*(<\S[^<>]*>))+[^<]*$")


class Description:
    r"""Event description compatibility.

    This includes HTML and text description.

    Example:

    .. code-block:: python

        >>> from icalendar import Event
        >>> from icalendar_compatibility import Description
        >>> event_string = '''
        ... BEGIN:VEVENT
        ... SUMMARY:Eventwith HTML
        ... DTSTART;TZID=Europe/London:20240414T090000
        ... DESCRIPTION;ALTREP="data:text/html,%3Ch1%3EKnow%20This%20Heading!%3C%2Fh1%3
        ...  E%3Cbr%3EPlease%20have%20a%20look%20at%20this%20website%3A%3Cbr%3E%3Ca%20hr
        ...  ef%3D%22https%3A%2F%2Fopen-web-calendar.quelltext.eu%2Ftemplates%2F%22%3EEx
        ...  amples%3C%2Fa%3E%3Cbr%3E%F0%9F%99%82%3Col%3E%3Cli%3Eone%3C%2Fli%3E%3Cli%3Et
        ...  wo%3C%2Fli%3E%3C%2Fol%3EAnd%20consider%20this%3A%3Cul%3E%3Cli%3Ea%20bullet%
        ...  20point%3C%2Fli%3E%3Cli%3Eand%20another%20bullet%20point%3C%2Fli%3E%3C%2Ful
        ...  %3E%3Cp%3E%3Cb%3Ebold%3C%2Fb%3E%2C%20%3Cu%3Eunderlined%3C%2Fu%3E%20and%20%3
        ...  Ci%3Eitalic%3C%2Fi%3E%20work!%3C%2Fp%3E%3Cpre%3Ecode%3Cbr%3Ecode%3Cbr%3Ecod
        ...  e%3C%2Fpre%3E":Know This Heading!\\n\\nPlease have a look at this website:\\nE
        ...  xamples\\n🙂\\n\\n    one\\n    two\\n\\nAnd consider this:\\n\\n    a bullet poi
        ...  nt\\n    and another bullet point\\n\\nbold\\, underlined and italic work!\\n\\nc
        ...  ode\\ncode\\ncode
        ... END:VEVENT
        ... '''
        >>> event = Event.from_ical(event_string)
        >>> description = Description(event)
        >>> description.html[:49]
        '<h1>Know This Heading!</h1><br>Please have a look'
        >>> description.text[:38]
        'Know This Heading!\n\nPlease have a look'

    .. note::

        Please note that sometimes there is an HTML description but not a text
        description.

    .. code-block:: python

        >>> event_string = '''
        ... BEGIN:VEVENT
        ... SUMMARY:Eventwith HTML
        ... DTSTART;TZID=Europe/London:20240414T090000
        ... DESCRIPTION:<p>HTML description</p>
        ... END:VEVENT
        ... '''
        >>> event = Event.from_ical(event_string)
        >>> description = Description(event)
        >>> description.html
        '<p>HTML description</p>'
        >>> description.text
        ''

    """

    def __init__(self, event: Event):
        """Create a new Description extraction for the event."""
        self._event = event

    @property
    def raw_description(self) -> vText:
        """The raw desctiption from the event.."""
        return self._event.get("DESCRIPTION", vText(""))

    @property
    def html(self) -> str:
        r"""The event description as HTML.

        The HTML description is stored in different places.
        This extracts the HTML description.

        """
        description = self.raw_description
        # print(self._event)
        altrep = description.params.get("ALTREP")
        # print("altrep", description, description.params)
        if altrep is not None and "," in altrep:
            data, content = altrep.split(",", 1)
            if data == "data:text/html":
                # Thunderbird html
                return unquote(content)
        # CMS4Schools.com
        altdesc = self._event.get("X-ALT-DESC")
        if (
            altdesc is not None
            and hasattr(altdesc, "params")
            and altdesc.params.get("FMTTYPE") == "text/html"
        ):
            return str(altdesc)
        if self.this_could_be_html(description):
            return str(description)
        return ""

    @property
    def text(self) -> str:
        """The text description of the event or an empty string."""
        description = str(self.raw_description)
        if description == self.html:
            return ""
        return str(description)

    @staticmethod
    def this_could_be_html(description: str):
        """Wether this is possibly HTML."""
        return bool(RE_HTML_TAGS_PROPER.match(description))


__all__ = ["Description"]



# icalendar_compatibility
# Copyright (C) 2025  Nicco Kunzmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see
# <https://www.gnu.org/licenses/>.
"""Create compatibility access for icalendar components."""

from .description import Description
from .location import Location, LocationSpec
from .version import __version__, __version_tuple__, version, version_tuple

__all__ = [
    "Description",
    "Location",
    "LocationSpec",
    "__version__",
    "__version_tuple__",
    "version",
    "version_tuple",
]



# icalendar_compatibility
# Copyright (C) 2025  Nicco Kunzmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see
# <https://www.gnu.org/licenses/>.
try:
    from ._version import __version__, __version_tuple__, version, version_tuple
except ModuleNotFoundError:
    __version__ = version = "0.0dev0"
    __version_tuple__ = version_tuple = (0, 0, "dev0")
import sys
from importlib.metadata import version as get_version

cli_version = f"""{__version__}

Components:
"""
modules = [
    "recurring-ical-events",
    "icalendar",
    "python-dateutil",
    "pytz",
    "click",
    "tzdata",
    "x-wr-timezone",
]
modules.sort()
for module in modules:
    try:
        cli_version += f"{module}: {get_version(module)}\n"
    except ModuleNotFoundError:  # noqa: PERF203
        cli_version += f"{module}: not installed\n"

cli_version += f"""
Python: {sys.version}"""

__all__ = [
    "__version__",
    "__version_tuple__",
    "cli_version",
    "version",
    "version_tuple",
]



# icalendar_compatibility
# Copyright (C) 2025  Nicco Kunzmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see
# <https://www.gnu.org/licenses/>.
"""This file tests the source code provided by the documentation.

See
- doctest documentation: https://docs.python.org/3/library/doctest.html
- Issue 443: https://github.com/collective/icalendar/issues/443

This file should be tests, too:

    >>> print("Hello World!")
    Hello World!

"""

import doctest
import importlib
import pathlib

import pytest

HERE = pathlib.Path(__file__).parent
PROJECT_PATH = HERE.parent

PYTHON_FILES = list(PROJECT_PATH.rglob("*.py"))
DOCS_PATH = PROJECT_PATH.parent / "docs"
RST_FILES = list(DOCS_PATH.rglob("*.rst")) + [PROJECT_PATH.parent / "README.rst"]

MODULE_NAMES = [
    "icalendar_compatibility",
]


@pytest.mark.parametrize("module_name", MODULE_NAMES)
def test_docstring_of_python_file(module_name):
    """This test runs doctest on the Python module."""
    module = importlib.import_module(module_name)
    test_result = doctest.testmod(module, name=module_name)
    assert test_result.failed == 0, f"{test_result.failed} errors in {module_name}"


# This collection needs to exclude .tox and other subdirectories
DOCUMENT_PATHS = RST_FILES + PYTHON_FILES


def test_this_file_is_also_tested():
    """Check we recurse."""
    paths = list(map(str, DOCUMENT_PATHS))
    print("Documents:\n", "\n".join(paths))
    assert __file__ in paths


def doctest_print(obj):
    """doctest print"""
    if isinstance(obj, bytes):
        obj = obj.decode("UTF-8")
    print(str(obj).strip().replace("\r\n", "\n").replace("\r", "\n"))


@pytest.mark.parametrize("document", DOCUMENT_PATHS)
def test_documentation_file(document):
    """This test runs doctest on a documentation file.

    functions are also replaced to work.
    """
    test_result = doctest.testfile(
        str(document),
        module_relative=False,
        globs={"print": doctest_print},
        raise_on_error=False,
    )
    assert test_result.failed == 0, f"{test_result.failed} errors in {document.name}"



# icalendar_compatibility
# Copyright (C) 2025  Nicco Kunzmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see
# <https://www.gnu.org/licenses/>.

from pathlib import Path

import pytest
from icalendar import Calendar

from icalendar_compatibility import Location, LocationSpec
from icalendar_compatibility.description import Description

HERE = Path(__file__).parent
EVENTS = HERE / "events"

for file in EVENTS.iterdir():
    if file.suffix.lower() == ".ics":

        def _fixture():
            raise NotImplementedError("Do not know what to do with this.")

        if "location" in file.stem:

            def _fixture(location_spec: LocationSpec, path: Path = file) -> Location:
                """Create an event adapter."""
                cal = Calendar.from_ical(path.read_text())
                assert (
                    len(cal.events) == 1
                ), f"We need one event only, not {len(cal.events)} in {path.stem}"
                return Location(cal.events[0], location_spec)

        elif "description" in file.stem:

            def _fixture(path: Path = file) -> Description:
                """Create an event adapter."""
                cal = Calendar.from_ical(path.read_text())
                assert (
                    len(cal.events) == 1
                ), f"We need one event only, not {len(cal.events)} in {path.stem}"
                return Description(cal.events[0])

        locals()[file.stem] = pytest.fixture(_fixture)
    else:
        raise ValueError(f"Do not know what to do with {file}")


@pytest.fixture(
    params=[
        LocationSpec.for_openstreetmap_org,
        LocationSpec.for_bing_com,
        LocationSpec.for_google_co_uk,
    ]
)
def location_spec(request) -> LocationSpec:
    """The default spec."""
    return request.param()



# icalendar_compatibility
# Copyright (C) 2025  Nicco Kunzmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see
# <https://www.gnu.org/licenses/>.
"""Event location computation."""

from __future__ import annotations

import re
from dataclasses import dataclass
from typing import Optional
from urllib.parse import quote

from icalendar import Event, vGeo, vText

GEO_MATCH = re.compile(r"^(?P<lat>[-+]?\d*\.?\d+)\s*,\s*(?P<lon>[-+]?\d*\.?\d+)$")

URL_REGEX = re.compile(
    r"https?:\/\/(www\.)?[-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,4}\b([-a-zA-Z0-9@:%_\+.~#?&//=]*)"
)  # see https://stackoverflow.com/a/3809435/1320237


@dataclass
class LocationSpec:
    """Specification for event locations.

    Attributes:
        zoom : int
            Zoom level for geo_url and search_url
        geo_url : str
            A url template when the geo location is given.
            At least {lat} and {lon} are required.
        text_url : str
            A url template when the location is given as text.
            At leat {location} is required.

    Examples:

        >>> from icalendar_compatibility import LocationSpec
        >>> spec = LocationSpec.for_openstreetmap_org()

    """

    geo_url: str
    text_url: str
    zoom: int = 16

    @classmethod
    def for_openstreetmap_org(cls, **kw) -> LocationSpec:
        """Spec for https://openstreetmap.org"""
        return cls(
            geo_url="https://www.openstreetmap.org/#map={zoom}/{lat}/{lon}",
            text_url="https://www.openstreetmap.org/search?query={location}",
            **kw,
        )

    @classmethod
    def for_bing_com(cls, **kw) -> LocationSpec:
        """Spec for https://www.bing.com/maps"""
        return cls(
            geo_url="https://www.bing.com/maps?brdr=1&cp={lat}%7E{lon}&lvl={zoom}",
            text_url="https://www.bing.com/maps?q={location}&lvl={zoom}",
            **kw,
        )

    @classmethod
    def for_google_com(cls, **kw) -> LocationSpec:
        """Spec for https://www.google.com/maps"""
        return cls(
            geo_url="https://www.google.com/maps/@{lat},{lon},{zoom}z",
            text_url="https://www.google.com/maps/search/{location}",
            **kw,
        )

    @classmethod
    def for_google_co_uk(cls, **kw) -> LocationSpec:
        """Spec for https://www.google.co.uk/maps"""
        return cls(
            geo_url="https://www.google.co.uk/maps/@{lat},{lon},{zoom}z",
            text_url="https://www.google.co.uk/maps/search/{location}",
            **kw,
        )

    @classmethod
    def for_no_url(cls) -> LocationSpec:
        """Return a spec that creates empty URLs always."""
        return cls(geo_url="", text_url="")

    def get_geo_url(self, *, lat: float, lon: float, zoom: Optional[int] = None) -> str:
        """Get the url for a geo location."""
        return self.geo_url.format(
            lat=lat, lon=lon, zoom=self.zoom if zoom is None else zoom
        )

    def get_text_url(self, location: str, zoom: Optional[int] = None) -> str:
        """Get the url for a text location."""
        return self.text_url.format(
            location=self.quote(location), zoom=self.zoom if zoom is None else zoom
        )

    @staticmethod
    def quote(string: str):
        """Quote a string to fit into any place in a URL.

        We also replace the . as .. has a special meaning.
        """
        return quote(string, safe="").replace(".", "%2e")


class Location:
    """The location of an event.

    Attributes:
        text : str
            The text of the location.
        url : str
            The url of the event location.
            This considers geo information, text and more.

    Examples:

        >>> from icalendar_compatibility import Location, LocationSpec
        >>> from icalendar import Event
        >>> event_string = '''
        ... BEGIN:VEVENT
        ... SUMMARY:Event in Mountain View with Geo link
        ... DTSTART:20250115T150000Z
        ... LOCATION:Mountain View, Santa Clara County, Kalifornien, Vereinigte Staaten von Amerika
        ... GEO:37.386013;-122.082932
        ... END:VEVENT
        ... '''
        >>> event = Event.from_ical(event_string)
        >>> location = Location(event, LocationSpec.for_bing_com())
        >>> print(location.text)
        Mountain View, Santa Clara County, Kalifornien, Vereinigte Staaten von Amerika
        >>> print(location.url)
        https://www.bing.com/maps?brdr=1&cp=37.386013%7E-122.082932&lvl=16


    """

    def __init__(self, event: Event, spec: Optional[LocationSpec] = None):
        """Create a new location adapter.

        Args:
            event : Event
                The event to adapt.
            spec : LocationSpec
                The specification to use.
                By default we use OpenStreetMap.
        """
        self._event = event
        self._spec = LocationSpec.for_openstreetmap_org() if spec is None else spec

    @property
    def raw_text(self) -> vText:
        r"""The raw event text of the location.

        :rfc:`5545`::

            LOCATION:Conference Room - F123\, Bldg. 002
        """
        return self._event.get("LOCATION", vText(""))

    @property
    def raw_altrep(self) -> str:
        r"""The alternative representation according to RFC5545.

        :rfc:`5545`::

            LOCATION;ALTREP="http://xyzcorp.com/conf-rooms/f123.vcf":
            Conference Room - F123\, Bldg. 002
        """
        return self.raw_text.params.get("ALTREP", "")

    @property
    def raw_geo(self) -> Optional[vGeo]:
        """The raw geo location.

        :rfc:`5545`::

            GEO:37.386013;-122.082932
        """
        geo = self._event.get("GEO")
        if geo is not None:
            return geo
        match = GEO_MATCH.match(self.raw_text)
        if match:
            return vGeo((float(match.group("lat")), float(match.group("lon"))))
        return None

    @property
    def text(self) -> str:
        """The location text.

        Returns: str
            The text or an empty string if we have no location data.
        """
        return str(self.raw_text)

    @property
    def url(self) -> str:
        """The location url.

        Returns: str
            The url or an empty string if we have no location data.
        """
        if self.raw_altrep:
            return self.raw_altrep
        lon, lat = self.lon, self.lat
        if lon is None or lat is None:
            text = self.text
            if text == "":
                return ""
            for url in URL_REGEX.finditer(text):
                return url.group()
            return self._spec.get_text_url(location=self.text, zoom=self.zoom)
        return self._spec.get_geo_url(lat=lat, lon=lon, zoom=self.zoom)

    @property
    def lon(self) -> Optional[float]:
        """The longitude of the location.

        Returns: float
            The longitude or None if we have no location data.
        """
        return self.raw_geo.longitude if self.raw_geo else None

    @property
    def lat(self) -> Optional[float]:
        """The latitude of the location.

        Returns: float
            The longitude or None if we have no location data.
        """
        return self.raw_geo.latitude if self.raw_geo else None

    @property
    def zoom(self) -> int:
        """The zoom level of the location.

        Returns: int
            The zoom level of the location.
        """
        return self._spec.zoom

    @property
    def spec(self) -> LocationSpec:
        """The location spec we use."""
        return self._spec

    @spec.setter
    def spec(self, new_spec: LocationSpec):
        """Set the location spec."""
        self._spec = new_spec


__all__ = ["Location", "LocationSpec"]



# icalendar_compatibility
# Copyright (C) 2025  Nicco Kunzmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see
# <https://www.gnu.org/licenses/>.
from urllib.parse import quote, unquote

import pytest
from icalendar import Event

from icalendar_compatibility import Location, LocationSpec


def test_altrep(location_altrep: Location):
    """Check the usage of altrep."""
    assert location_altrep.url == "https://www.openstreetmap.org/relation/62422"
    assert location_altrep.text == "Berlin"


def test_geo_in_location_text(location_geo_misplaced: Location):
    """Check the link is as expected."""
    assert location_geo_misplaced.text == "50.1075325012207, 14.2693090438843"


def test_geo_in_location_geo(
    location_geo_misplaced: Location, location_spec: LocationSpec
):
    """Check the link is as expected."""
    assert location_geo_misplaced.lat == 50.1075325012207
    assert location_geo_misplaced.lon == 14.2693090438843
    assert location_geo_misplaced.zoom == location_spec.zoom


def test_geo_location_given(location_geo: Location, location_spec: LocationSpec):
    """Check that we have the namme and a URL."""
    assert location_geo.lat == 37.386013
    assert location_geo.lon == -122.082932
    assert location_geo.zoom == location_spec.zoom


def test_we_generate_the_link_from_the_location_text(
    location_text: Location, location_spec: LocationSpec
):
    """We generate a link."""
    assert location_text.text == "Berlin"
    assert location_text.url == location_spec.get_text_url(
        location="Berlin", zoom=location_spec.zoom
    )


def test_link_from_geo_1(location_geo: Location, location_spec: LocationSpec):
    assert (
        location_geo.text
        == "Mountain View, Santa Clara County, Kalifornien, Vereinigte Staaten von Amerika"
    )
    assert location_geo.url == location_spec.get_geo_url(
        lat=37.386013, lon=-122.082932, zoom=location_spec.zoom
    )


def test_link_from_geo_2(location_geo_misplaced: Location, location_spec: LocationSpec):
    assert location_geo_misplaced.url == location_spec.get_geo_url(
        lat=50.1075325012207, lon=14.2693090438843
    )


def test_get_geo_url():
    assert (
        LocationSpec(geo_url="{lat}x{lon}", text_url="").get_geo_url(lat=50, lon=14)
        == "50x14"
    )
    assert (
        LocationSpec(geo_url="{lat}x{lon}, z={zoom}", text_url="").get_geo_url(
            lat=50, lon=14
        )
        == "50x14, z=16"
    )
    assert (
        LocationSpec(geo_url="{lat}x{lon}, z={zoom}", text_url="").get_geo_url(
            lat=5, lon=1, zoom=4
        )
        == "5x1, z=4"
    )


def test_Location_with_no_location(no_location: Location):
    """Check that the location is empty."""
    assert no_location.text == ""
    assert no_location.url == ""


def test_url_in_location_counts_as_url(location_link):
    """If we have a URL in the location, we use the first URL as the location URL."""
    assert location_link.url == "https://www.berlin.de/"
    assert location_link.text == " We meet in https://www.berlin.de/ "


@pytest.mark.parametrize(
    ("forbidden_content"),
    [
        "/../../../",
        "?asd=asd",
        "..",
        " asd ",
        "?a",
        "\r\n",
    ],
)
def test_location_information_is_inserted_url_escaped(forbidden_content):
    """We should test that the / is escaped and that characters like ? do not appear. They would break the URL.

    Escape:
    - spaces and other characters
    - dots
    - slash
    - ?
    """
    event = Event()
    event.add("LOCATION", forbidden_content)
    location = Location(event)
    assert forbidden_content not in location.url
    quoted = LocationSpec.quote(forbidden_content)
    assert quoted in location.url


INVALID_EVENT_GEO = """
BEGIN:VEVENT
GEO:12/../../..,123
END:VEVENT
"""


def test_geo_location_is_also_escaped():
    """The geo location should not work with bad input."""
    event = Event.from_ical(INVALID_EVENT_GEO)
    location = Location(event)
    assert location.url == ""
    assert location.lat is None
    assert location.lon is None


def test_location_with_unicode():
    """Test that the location works with unicode."""
    event = Event()
    name = "České Budějovice"
    event.add("LOCATION", name)
    location = Location(event)
    assert name not in location.url
    assert quote(name) in location.url


def test_replace_spec_with_empty_spec(location_geo: Location):
    """The spec can be empty."""
    location_geo.spec = LocationSpec.for_no_url()
    assert location_geo.url == ""


def test_quote_unquote():
    """Make sure we can quote and unquote all the strings."""
    for i in range(256):
        c = chr(i)
        assert unquote(LocationSpec.quote(c)) == c


@pytest.mark.parametrize("char", ["%", "=", "?", "/", "&", ".", "<", ">", '"', "'"])
def test_quote_certain_characters(char):
    quoted = LocationSpec.quote(char)
    assert quoted.startswith("%")
    assert quoted != char



# icalendar_compatibility
# Copyright (C) 2025  Nicco Kunzmann
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see
# <https://www.gnu.org/licenses/>.
"""Test getting the description of calendar files."""

import pytest

from icalendar_compatibility.description import Description


def test_no_description_is_there(no_description: Description):
    """We should have no description."""
    assert no_description.text == ""
    assert no_description.html == ""


def test_html_description_is_recognized(description_html: Description):
    """Check that we recognize the HTML description."""
    assert (
        description_html.html
        == 'Marceline Car Show - <a href="https://www.MarcelineCarShow.com">https://www.MarcelineCarShow.com</a>'
    )
    assert description_html.text == ""


@pytest.mark.parametrize(
    ("s", "is_html"),
    [
        ("<", False),
        (">", False),
        ("<a>", True),
        (
            'Marceline Car Show - <a href="https://www.MarcelineCarShow.com">https://www.MarcelineCarShow.com</a>',
            True,
        ),
        (
            "jhaskjfhajkfh jahskjd haskhk asdkjahs, asjdhajsjf../asdaskjfk < nannana  kajskldhlkas\naskjdlkasjld > akshdj",
            False,
        ),
    ],
)
def test_html_probability(s, is_html):
    """Test the HTML recognition."""
    assert Description.this_could_be_html(s) == is_html


def test_altrep(description_altrep_html: Description):
    """Check that we can have both: text and html."""
    assert (
        description_altrep_html.text
        == "Know This Heading!\n\nPlease have a look at this website:\nExamples\n🙂\n\n    one\n    two\n\nAnd consider this:\n\n    a bullet point\n    and another bullet point\n\nbold, underlined and italic work!\n\ncode\ncode\ncode"
    )
    assert (
        description_altrep_html.html
        == '<h1>Know This Heading!</h1><br>Please have a look at this website:<br><a href="https://open-web-calendar.quelltext.eu/templates/">Examples</a><br>🙂<ol><li>one</li><li>two</li></ol>And consider this:<ul><li>a bullet point</li><li>and another bullet point</li></ul><p><b>bold</b>, <u>underlined</u> and <i>italic</i> work!</p><pre>code<br>code<br>code</pre>'
    )


def test_recognize_x_alt_desc(x_alt_description: Description):
    """Recognize the alternative description."""
    assert (
        x_alt_description.html
        == "<p>Estudio</p><p><em>Presentan: Tomás Lambertini </em>y <em>Pablo Porcel</em></p><p><em>Habrá café y alimentos biodinámicos. Aporte económico voluntario.</em></p>"
    )
    assert (
        x_alt_description.text
        == "Estudio\n\nPresentan: Tomás Lambertini y Pablo Porcel\n\nHabrá café y alimentos biodinámicos. Aporte económico voluntario."
    )
